@@ -1,14 +1,15 @@
-use cargo_test_support::compare::assert_ui;
 use cargo_test_support::Project;
+use cargo_test_support::compare::assert_ui;
+use cargo_test_support::file;
 
-use crate::init_registry;
 use crate::CargoCommand;
-use cargo_test_support::curr_dir;
+use crate::init_registry;
+use cargo_test_support::current_dir;
 
 #[cargo_test]
 fn case() {
     init_registry();
-    let project = Project::from_template(curr_dir!().join("in"));
+    let project = Project::from_template(current_dir!().join("in"));
     let project_root = project.root();
     let cwd = project_root.join("one");
 
@@ -18,8 +19,8 @@ fn case() {
         .current_dir(cwd)
         .assert()
         .success()
-        .stdout_matches_path(curr_dir!().join("stdout.log"))
-        .stderr_matches_path(curr_dir!().join("stderr.log"));
+        .stdout_eq(file!["stdout.term.svg"])
+        .stderr_eq(file!["stderr.term.svg"]);
 
-    assert_ui().subset_matches(curr_dir!().join("out"), &project_root);
+    assert_ui().subset_matches(current_dir!().join("out"), &project_root);
 }
