@@ -4,8 +4,10 @@ use crate::connect_raw::connect_raw;
 use crate::connect_socket::connect_socket;
 use crate::tls::MakeTlsConnect;
 use crate::{Client, Config, Connection, Error, SimpleQueryMessage, Socket};
-use futures_util::{future, pin_mut, Future, FutureExt, Stream};
+use futures_util::{FutureExt, Stream};
 use rand::seq::SliceRandom;
+use std::future::{self, Future};
+use std::pin::pin;
 use std::task::Poll;
 use std::{cmp, io};
 use tokio::net;
@@ -161,18 +163,18 @@ where
     let (mut client, mut connection) = connect_raw(socket, tls, has_hostname, config).await?;
 
     if config.target_session_attrs != TargetSessionAttrs::Any {
-        let rows = client.simple_query_raw("SHOW transaction_read_only");
-        pin_mut!(rows);
+        let mut rows = pin!(client.simple_query_raw("SHOW transaction_read_only"));
 
-        let rows = future::poll_fn(|cx| {
-            if connection.poll_unpin(cx)?.is_ready() {
-                return Poll::Ready(Err(Error::closed()));
-            }
+        let mut rows = pin!(
+            future::poll_fn(|cx| {
+                if connection.poll_unpin(cx)?.is_ready() {
+                    return Poll::Ready(Err(Error::closed()));
+                }
 
-            rows.as_mut().poll(cx)
-        })
-        .await?;
-        pin_mut!(rows);
+                rows.as_mut().poll(cx)
+            })
+            .await?
+        );
 
         loop {
             let next = future::poll_fn(|cx| {
