@@ -53,7 +53,8 @@ impl<In: Transport> Connector<In> for TestConnector {
             buffers,
             tx: tx1,
             rx: SyncReceiver(Mutex::new(rx2)),
-            connected: true,
+            connected_tx: true,
+            connected_rx: true,
         };
 
         Ok(Some(Either::B(transport)))
@@ -151,12 +152,12 @@ fn setup_default_handlers(handlers: &mut Vec<TestHandler>) {
                 "HTTP/1.1 200 OK\r\n\
                 Content-Type: text/html;charset=ISO-8859-1\r\n\
                 set-cookie: AEC=AVYB7cpadYFS8ZgaioQ17NnxHl1QcSQ_2aH2WEIg1KGDXD5kjk2HhpGVhfk; \
-                    expires=Mon, 14-Apr-2025 17:23:39 GMT; path=/; domain=.google.com; \
+                    expires=Mon, 14-Apr-2050 17:23:39 GMT; path=/; domain=.google.com; \
                     Secure; HttpOnly; SameSite=lax\r\n\
                 set-cookie: __Secure-ENID=23.SE=WaDe-mOBoV2nk-IwHr73boNt6dYcjzQh1X_k8zv2UmUXBL\
                     m80a3pzLJyx1N1NOqBxDDOR8OJyvuNYw5phFf0VnbqzVtcKPijo2FY8O_vymzyc7x2VwFhGlgU\
                     WXSWYinjWL7Zvz_EOcA4kfnEXweW5ZDzLrvaLuBIrz5CA_-454AMIXpDiZAVPChCawbkzMptAr\
-                    lMTikkon2EQVXsicqq1XnrMEMPZR5Ld2JC6lpBM8A; expires=Sun, 16-Nov-2025 09:41:57 \
+                    lMTikkon2EQVXsicqq1XnrMEMPZR5Ld2JC6lpBM8A; expires=Sun, 16-Nov-2050 09:41:57 \
                     GMT; path=/; domain=.google.com; Secure; HttpOnly; SameSite=lax\r\n\
                 \r\n\
                 ureq test server here"
@@ -192,6 +193,21 @@ fn setup_default_handlers(handlers: &mut Vec<TestHandler>) {
         handlers,
     );
 
+    maybe_add(
+        TestHandler::new("/bytes/200000000", |_uri, _req, w| {
+            write!(
+                w,
+                "HTTP/1.1 200 OK\r\n\
+                Content-Type: application/octet-stream\r\n\
+                Content-Length: 100\r\n\
+                \r\n"
+            )?;
+            // We don't actually want 200MB of data in memory.
+            write!(w, "{}", "1".repeat(100))
+        }),
+        handlers,
+    );
+
     maybe_add(
         TestHandler::new("/get", |_uri, req, w| {
             write!(
@@ -210,6 +226,24 @@ fn setup_default_handlers(handlers: &mut Vec<TestHandler>) {
         handlers,
     );
 
+    maybe_add(
+        TestHandler::new("/?query=foo", |_uri, req, w| {
+            write!(
+                w,
+                "HTTP/1.1 200 OK\r\n\
+                Content-Type: application/json\r\n\
+                Content-Length: {}\r\n\
+                \r\n",
+                HTTPBIN_GET.len()
+            )?;
+            if req.method() != Method::HEAD {
+                w.write_all(HTTPBIN_GET.as_bytes())?;
+            }
+            Ok(())
+        }),
+        handlers,
+    );
+
     maybe_add(
         TestHandler::new("/head", |_uri, _req, w| {
             write!(
@@ -321,7 +355,7 @@ fn setup_default_handlers(handlers: &mut Vec<TestHandler>) {
                 "HTTP/1.1 302 OK\r\n\
                 Location: /get\r\n\
                 set-cookie: AEC=AVYB7cpadYFS8ZgaioQ17NnxHl1QcSQ_2aH2WEIg1KGDXD5kjk2HhpGVhfk; \
-                    expires=Mon, 14-Apr-2025 17:23:39 GMT; path=/; domain=.google.com; \
+                    expires=Mon, 14-Apr-2050 17:23:39 GMT; path=/; domain=.google.com; \
                     Secure; HttpOnly; SameSite=lax\r\n\
                 " // deliberately omit final \r\n
             )
@@ -392,6 +426,73 @@ fn setup_default_handlers(handlers: &mut Vec<TestHandler>) {
         handlers,
     );
 
+    maybe_add(
+        TestHandler::new("/1chunk-abort", |_uri, _req, w| {
+            write!(
+                w,
+                "HTTP/1.1 200 OK\r\n\
+                Transfer-Encoding: chunked\r\n\
+                \r\n\
+                2\r\n\
+                OK\r\n\
+                0\r\n<hangup>",
+            )?;
+            Ok(())
+        }),
+        handlers,
+    );
+
+    maybe_add(
+        TestHandler::new("/2chunk-abort", |_uri, _req, w| {
+            write!(
+                w,
+                "HTTP/1.1 200 OK\r\n\
+                Transfer-Encoding: chunked\r\n\
+                \r\n\
+                2\r\n\
+                OK\r\n\
+                0\r\n\
+                \r<hangup>", // missing \n
+            )?;
+            Ok(())
+        }),
+        handlers,
+    );
+
+    maybe_add(
+        TestHandler::new("/3chunk-abort", |_uri, _req, w| {
+            write!(
+                w,
+                "HTTP/1.1 200 OK\r\n\
+                Transfer-Encoding: chunked\r\n\
+                \r\n\
+                2\r\n\
+                OK\r\n\
+                0\r\n\
+                \r\n<hangup>",
+            )?;
+            Ok(())
+        }),
+        handlers,
+    );
+
+    maybe_add(
+        TestHandler::new("/4chunk-abort", |_uri, _req, w| {
+            write!(
+                w,
+                "HTTP/1.1 200 OK\r\n\
+                Transfer-Encoding: chunked\r\n\
+                \r\n\
+                2\r\n\
+                OK\r\n\
+                0\r\n\
+                \r\n",
+            )?;
+            Ok(())
+        }),
+        handlers,
+    );
+
     #[cfg(feature = "charset")]
     {
         let (cow, _, _) =
@@ -471,7 +572,7 @@ impl io::Read for RxRead {
             Ok(v) => v,
             Err(_) => return Ok(0), // remote side is gone
         };
-        assert!(buf.len() > v.len());
+        assert!(buf.len() >= v.len(), "{} > {}", buf.len(), v.len());
         let max = buf.len().min(v.len());
         buf[..max].copy_from_slice(&v[..]);
         Ok(max)
@@ -537,7 +638,8 @@ pub(crate) struct TestTransport {
     buffers: LazyBuffers,
     tx: mpsc::SyncSender<Vec<u8>>,
     rx: SyncReceiver<Vec<u8>>,
-    connected: bool,
+    connected_tx: bool,
+    connected_rx: bool,
 }
 
 impl Transport for TestTransport {
@@ -548,25 +650,45 @@ impl Transport for TestTransport {
     fn transmit_output(&mut self, amount: usize, _timeout: NextTimeout) -> Result<(), Error> {
         let output = &self.buffers.output()[..amount];
         if self.tx.send(output.to_vec()).is_err() {
-            self.connected = false;
+            self.connected_tx = false;
         }
         Ok(())
     }
 
     fn await_input(&mut self, timeout: NextTimeout) -> Result<bool, Error> {
+        if !self.connected_rx {
+            return Err(Error::Io(io::Error::new(
+                io::ErrorKind::UnexpectedEof,
+                "test server is not connected",
+            )));
+        }
+
         let input = self.buffers.input_append_buf();
-        let buf = match self.rx.recv_timeout(timeout.after) {
+        let mut buf = match self.rx.recv_timeout(timeout.after) {
             Ok(v) => v,
             Err(RecvTimeoutError::Timeout) => return Err(Error::Timeout(timeout.reason)),
             Err(RecvTimeoutError::Disconnected) => {
                 trace!("Test server disconnected");
-                self.connected = false;
+                self.connected_rx = false;
                 return Err(Error::Io(io::Error::new(
                     io::ErrorKind::UnexpectedEof,
                     "test server disconnected",
                 )));
             }
         };
+
+        let maybe_hangup = buf
+            .windows(HANGUP.len())
+            .enumerate()
+            .find(|(_, w)| *w == HANGUP)
+            .map(|(pos, _)| pos);
+
+        if let Some(pos) = maybe_hangup {
+            debug!("TEST: Found <hangup>");
+            buf.drain(pos..);
+            self.connected_rx = false;
+        }
+
         assert!(input.len() >= buf.len());
         let max = input.len().min(buf.len());
         input[..max].copy_from_slice(&buf[..]);
@@ -575,7 +697,7 @@ impl Transport for TestTransport {
     }
 
     fn is_open(&mut self) -> bool {
-        self.connected
+        self.connected_tx
     }
 
     fn is_tls(&self) -> bool {
@@ -584,6 +706,8 @@ impl Transport for TestTransport {
     }
 }
 
+const HANGUP: &[u8] = b"<hangup>";
+
 // Workaround for std::mpsc::Receiver not being Sync
 struct SyncReceiver<T>(Mutex<Receiver<T>>);
 
